unit Keysnoo_;
{
 Title     : Keysnooper
 Function  : Macro recorder example - receive keystrokes before the active
             application does. For example: You are working in Word, if you
             press F11 or F12 you want YOUR application to do something.
 Info:       a DLL is used to trap the keys. See keydll.pas
 Difficulty: experienced programmers
 By:         ing. S.P.Settels,
             e-mail: settels@pi.net
             delphi page: http://www.pi.net/~settels/delphi
}
interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls;

type
  EDLLError = class(Exception);

  tActivateKeySnooper = procedure(CurrentTask : thandle; messageno : word);

  TForm1 = class(TForm)
    Label1: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
  public
    dllhandle : thandle;
    ActivateKeySnooper : tActivateKeySnooper;
    procedure AppMessage(var Msg: TMsg; var Handled: Boolean);
  end;

var
  Form1: TForm1;

implementation

{$R *.DFM}

procedure TForm1.AppMessage(var Msg: TMsg; var Handled: Boolean);
{handler for the new message}
begin
  if Msg.Message = wm_user + 1 then
  begin
    if Msg.lparam and $80000000 = $80000000 then  {bit 31 = "key-down"}
    if Msg.wparam = VK_F12 then showmessage('F12 intercepted!') else
    if Msg.wparam = VK_F11 then showmessage('F11 intercepted!') {else ...};

    handled := true;   {'ok, this message is handled'}
  end;
end;

{procedure ActivateKeySnooper(CurrentTask : thandle; messageno : word);
                                           far; external 'keydll.dll' index 1;
{The 'keysnooper' needs to know which message to send to whom}

procedure TForm1.FormCreate(Sender: TObject);
const
  DLLErrors : array[0..21] of string =
  ({0}'System was out of memory, executable file was corrupt, or relocations were invalid',
   {1}'',
   {2}'File was not found',
   {3}'Path was not found',
   {4}'',
   {5}'Attempt was made to dynamically link to a task, or there was a sharing or network-protection error',
   {6}'Library required separate data segments for each task',
   {7}'',
   {8}'There was insufficient memory to start the application',
   {9}'',
  {10}'Windows version was incorrect',
  {11}'Executable file was invalid. Either it was not a Windows application or there was an error in the .EXE image',
  {12}'Application was designed for a different operating system',
  {13}'Application was designed for MS-DOS 4.0',
  {14}'Type of executable file was unknown',
  {15}'Attempt was made to load a real-mode application (developed for an earlier version of Windows)',
  {16}'Attempt was made to load a second instance of an executable file containing multiple data segments '+
      'that were not marked read-only',
  {17}'',
  {18}'',
  {19}'Attempt was made to load a compressed executable file. The file must be decompressed before it can be loaded',
  {20}'Dynamic-link library (DLL) file was invalid. One of the DLLs required to run this application was corrupt',
  {21}'Application requires 32-bit extensions');

begin
  dllhandle := loadlibrary('.\KEYDLL.DLL');
  if dllhandle < 22{HINSTANCE_ERROR} then raise EDLLError.Create(DLLErrors[dllhandle]);

  @ActivateKeySnooper := getprocaddress(dllhandle,'SETTASK');
  if @ActivateKeySnooper = NIL
  then raise EDLLError.Create('Keysnooper will noy work: Procedure from KEYDLL.DLL not available')
  else ActivateKeySnooper(GetCurrentTask, wm_user + 1);
  {make it send it's messages to this application using message wm_user + 1}

  {install our own messagehandler}
  Application.OnMessage := AppMessage;
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
  if dllhandle >= HINSTANCE_ERROR then FreeLibrary(dllhandle);
end;

end.
